package de.fhkoeln.cvogt.android.draganddropandroid;

// Prof. Dr. Carsten Vogt
// Programmierung verteilter und mobiler Anwendungen
// (Programming Distributed and Mobile Applications)
// Cologne University of Applied Sciences
// Institute of Communications Engineering
// 11.09.2017

import android.app.Activity;
import android.graphics.Color;
import android.graphics.Point;
import android.graphics.drawable.GradientDrawable;
import android.util.Log;
import android.view.DragEvent;
import android.view.GestureDetector;
import android.view.Gravity;
import android.view.MotionEvent;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.TextView;

// Class for TextViews that can be dragged across the display and dropped on other views

public class DemoView extends TextView {

    Activity activity; // the activity displaying the view
    GestureDetector gestureDetector;  // gesture detector to detect long press events considered as the start of a drag operation

    // Constructor of the view

    DemoView(Activity activity, int text) {

        super(activity);
        this.activity = activity;

        // specify layout properties
        setText(text+"");
        setTextSize(60);
        setPadding(100,10,200,0);
        setTextColor(Color.BLACK);
        setGravity(Gravity.CENTER_HORIZONTAL);
        setBackgroundResource(R.drawable.rounded_corners);
        LinearLayout.LayoutParams lp = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT,LinearLayout.LayoutParams.WRAP_CONTENT);
        lp.setMargins(5, 5, 5, 5);
        setLayoutParams(lp);

        // set gesture detector to detect long press events and to inform the runtime system that dragging of this view has started
        gestureDetector = new GestureDetector(activity, new DemoViewGestureListener());
        gestureDetector.setIsLongpressEnabled(true);

    }

    public boolean onTouchEvent(MotionEvent ev) {
        // inform the gesture detector of the touch event
        gestureDetector.onTouchEvent(ev);
        return true;
    }

    // onDragEvent():
    // If another view is being dragged and this view (i.e. the view to which this onDrag() method belongs) is visible,
    // the runtime system will call this view's onDragEvent() to inform it of relevant dragging events.
    // The parameter ev provides detailed information about the event.

    public boolean onDragEvent(DragEvent ev) {

        // Determine the view that has caused the drag event.
        // In this program, ev.getLocalState() will return a reference to the dragged view, because such a reference has been passed to the startDrag() call
        // when the dragging has started (see below: DemoViewGestureListener > onLongPress(), 3rd parameter).
        // In general, any kind of data can be passed to startDrag() and then retrieved in onDragEvent() by ev.getLocalState().

        DemoView draggedView = (DemoView) ev.getLocalState();

        // react depending of the type of the drag event

        switch (ev.getAction()) {

            // a drag operation has started
            case DragEvent.ACTION_DRAG_STARTED:
                Log.v("INFO", ">>> " + this.getText() + " Drag Started ");
                break;

            // the dragged view has entered this view
            case DragEvent.ACTION_DRAG_ENTERED:
                Log.v("INFO",">>> "+this.getText()+" Drag Entered");
                // change the colour of this view to red
                ((GradientDrawable) getBackground()).setStroke(8, 0xffff0000);
                setTextColor(Color.RED);
                break;

            // the dragged view has left this view
            case DragEvent.ACTION_DRAG_EXITED:
                Log.v("INFO",">>>  "+this.getText()+" Drag Exited");
                // change the colour of this view back to black
                setTextColor(Color.BLACK);
                ((GradientDrawable) getBackground()).setStroke(8, 0xff000000);
                break;

            // the dragged view has been dropped onto this view
            case DragEvent.ACTION_DROP:
                Log.v("INFO", ">>>  " + this.getText() + " Dropped");
                // add the number of the dragged view to the number of this view and display it
                int valueThisView = Integer.parseInt(this.getText().toString());
                int valueDraggedView = Integer.parseInt(draggedView.getText().toString());
                setText((valueThisView+valueDraggedView)+"");
                // change the colour of this view back to black
                setTextColor(Color.BLACK);
                ((GradientDrawable) getBackground()).setStroke(8, 0xff000000);
                // remove the dragged view from the display
                ((LinearLayout)draggedView.getParent()).removeView(draggedView);
                break;

            // the current drag operation has ended
            case DragEvent.ACTION_DRAG_ENDED:
                Log.v("INFO", ">>>  " + this.getText() + " Drag Ended");
                break;

            default:
                break;

        }

        return true;  // the drag event has been successfully handled

    }

    private class DemoViewGestureListener extends GestureDetector.SimpleOnGestureListener {

        // for the necessity of onDown() and onUp(): see http://stackoverflow.com/questions/24326530/long-press-in-gesturedetector-also-fires-on-tap

        @Override
        public boolean onDown(MotionEvent event) { return true; }

        @Override
        public boolean onSingleTapUp(MotionEvent event) { return true; }

        // onLongPress() is called when the gesture detector of this view detects a long press event.

        @Override
        public void onLongPress(MotionEvent e) {

            Log.v("INFO", ">>> Long Press on "+DemoView.this.getText()+": Drag begins");

            // Create a shadow builder to specify what shall be displayed while the view is dragged.
            // A customized shadow builder is used here, because the standard shadow builder created by 'new View.DragShadowBuilder(DemoView.this);' will let the dragged view jump inappropriately when the drag operation starts.
            DemoDragShadowBuilder shadowBuilder = new DemoDragShadowBuilder(DemoView.this,e.getX(),e.getY());

            // Inform the runtime system that a dragging operation with this view has started.
            // The third parameter, a reference to this dragged view, can be retrieved by other views that are affected by the drag operation in their onDragEvent() methods (see above).
            startDrag(null, shadowBuilder, DemoView.this, 0);

            // Let the dragged view disappear from the display.
            DemoView.this.setVisibility(View.GONE);

        }

    }

    // Customized shadow builder to set the initial position of the drag shadow and its color when the dragging starts.
    // (see http://stackoverflow.com/questions/16463060/retrieving-precise-drop-position-of-android-view)

    private class DemoDragShadowBuilder extends View.DragShadowBuilder {

        private float x, y;

        public DemoDragShadowBuilder(View view, float x, float y) {  // (x,y) = position of dragged view when the dragging starts (see call of this constructor in onLongPress()).
            super(view);
            this.x = x+70;  // the initial position of the drag shadow shall be 70 pixels left to the position of the dragged view when the dragging starts
            this.y = y;
            ((TextView) view).setTextColor(Color.GRAY);  // the drag shadow shall appear in grey
        }

        @Override
        public void onProvideShadowMetrics(Point size, Point touchPoint) {
            super.onProvideShadowMetrics(size, touchPoint);
            touchPoint.set((int) x, (int) y);
        }

    }

}


